# Heltec E-Ink Modules

Third-party Arduino Library for **Heltec E-Ink Module** displays.<br />
Run-time graphics on Arduino UNO, using Adafruit-GFX (via [ZinggJM/GFX_Root](https://github.com/ZinggJM/GFX_Root)) 

This is made possible with *"paging".* 

**[Quickstart](/docs/Quickstart/Quickstart.md)** <br />
**[API](/docs/API.md)** <br />

[Huh? Paging?](#what-is-paging) <br />
[Supported Displays](#supported-displays) <br />
[Identifying Your Display](#identifying-your-display) <br />
[Wiring](#wiring) <br />
[Using the library](#using-the-library) <br />
[Configuration](#configuration-options) <br />
[Installation](#installation) <br />
[Acknowledgements](#acknowledgements)  <br />


## What is paging?

*In this context,* it means that the display is calculated and transmitted in several small pieces, rather than in one big memory-clogging lump. 

This means more calculations are performed, but less RAM is needed.
The resulting increase in calculation time ends up being insignificant, especially when compared to the slow refresh time of E-INK displays.

Most importantly, it is the what allows the *Arduino Uno* to work with these displays, which would otherwise overload the limited memory.



## Supported Displays

Heltec seem to reuse the same marketing names for different non-interchangeable displays. In this library, displays are referred to by the *model of the panel*.
You can determine the model by consulting the table below.

Alternatively, your can declare your panel using the [label printed on the flex connector](#3-declare-your-panel-instead-by-the-flex-connector-label).

If your Heltec E-ink display is not listed, please let me know.

<!-- Large table of display identification info -->

<table>
    <thead>
        <tr>
            <th>Model Name</th>
            <th align="center">Front Image</th>
            <th align="center">Rear Image</th>
            <th>Rear Label</th>
            <th>Colors</th>
            <th>Screen Protector</th>
            <th><a href="#3-declare-your-panel-instead-by-the-flex-connector-label">Flex Connector Label</a></th>
            <th>Resolution (px)</th>
            <th><a href="#fast-mode-partial-refresh">Fastmode</a></th>
        </tr>
    </thead>
    <tbody>
        <tr>
            <td><strong>DEPG0150BNS810</strong></td>
            <td align="center"><img alt="Front" src="Identification/DEPG0150BNS810-Front.jpg" /></td>
            <td align="center"><img alt="Rear" src="Identification/DEPG0150BNS810-Rear.jpg" /></td>
            <td>1.54&nbsp;Inch&nbsp;E&#8209;ink&nbsp;Display&nbsp;V2</td>
            <td>Black, White</td>
            <td>Red Tab</td>
            <td>FPC-8101</td>
            <td>200 x 200</td>
            <td>Yes</td>
        </tr>
        <tr>
            <td><strong>DEPG0154BNS800</strong></td>
            <td align="center"><img alt="Front" src="Identification/DEPG0154BNS800-Front.jpg" /></td>
            <td align="center"><img alt="Rear" src="Identification/DEPG0154BNS800-Rear.jpg" /></td>            
            <td>1.54&nbsp;Inch&nbsp;E&#8209;ink&nbsp;Display&nbsp;V2</td>
            <td>Black, White</td>
            <td>Red Tab</td>
            <td>FPC-7525</td>
            <td>152 x 152</td>
            <td>Yes</td>
        </tr>
        <tr>
            <td><strong>GDEP015OC1</strong>&nbsp;<sup>1</sup></td>
            <td align="center"><img alt="Front" src="Identification/GDEP015OC1-Front.jpg" /></td>
            <td align="center"><img alt="Rear" src="Identification/GDEP015OC1-Rear.jpg" /></td>            
            <td>1.54&nbsp;Inch&nbsp;E&#8209;ink&nbsp;Display&nbsp;V2</td>
            <td>Black, White</td>
            <td>Blue Tab</td>
            <td>HINK-E0154A05-A2</td>
            <td>200 x 200</td>
            <td>Yes</td>
        </tr>
        <tr>
            <td><strong>QYEG0213RWS800</strong></td>
            <td align="center"><img alt="Front" src="Identification/QYEG0213RWS800-Front.jpg" /></td>
            <td align="center"><img alt="Rear" src="Identification/QYEG0213RWS800-Rear.jpg" /></td>            
            <td>2.13&nbsp;Inch&nbsp;E&#8209;ink&nbsp;Display&nbsp;V2</td>
            <td>Black, White, Red</td>
            <td>Red Tab</td>
            <td>FPC-7528&nbsp;/<br>FPC-7528B</td>
            <td>250 x 122</td>
            <td>No</td>
        </tr>
        <tr>
            <td><strong>DEPG0290BNS75A</strong>&nbsp;</td>
            <td align="center"><img alt="Front" src="Identification/DEPG0290BNS75A-Front.jpg" /></td>
            <td align="center"><img alt="Rear" src="Identification/DEPG0290BNS75A-Rear.jpg" /></td>            
            <td>2.9&nbsp;Inch&nbsp;E&#8209;ink&nbsp;Display&nbsp;V2</td>
            <td>Black, White</td>
            <td>Red Tab</td>
            <td>FPC-750</td>
            <td>296 x 128</td>
            <td>Yes</td>
        </tr>
        <tr>
            <td><strong>DEPG0290BNS800</strong></td>
            <td align="center"><img alt="Front" src="Identification/DEPG0290BNS800-Front.jpg" /></td>
            <td align="center"><img alt="Rear" src="Identification/DEPG0290BNS800-Rear.jpg" /></td>            
            <td>2.9&nbsp;Inch&nbsp;E&#8209;ink&nbsp;Display&nbsp;V2</td>
            <td>Black, White</td>
            <td>Red Tab</td>
            <td>FPC-7519 rev.b</td>
            <td>296 x 128</td>
            <td>Yes</td>
        </tr>
        <tr>
            <td><strong>GDE029A1</strong></td>
            <td align="center"><img alt="Front" src="Identification/GDE029A1-Front.jpg" /></td>
            <td align="center"><img alt="Rear" src="Identification/GDE029A1-Rear.jpg" /></td>
            <td>2.9&nbsp;Inch&nbsp;E&#8209;ink&nbsp;Display&nbsp;V2</td>
            <td>Black, White</td>
            <td>Blue Tab</td>
            <td>SYX-1553</td>
            <td>296 x 128</td>
            <td>Yes</td>
        </tr>
    </tbody>
</table>

<!-- End of table: Display Identification -->

<sup>1</sup> Closest match. No official information available. <br />

## Identifying your display

### 1. Consult [the supported displays table](#supported-displays).
The identification data may be incorrect or incomplete. It is based on my own observation.

*or*

### 2. Upload the `identify_display` example sketch to your Arduino. 
With your display connected [through a level shifter](#wiring), press the reset button to test different models.

*or*

### 3. Declare your panel instead by the *flex connector label*

The E-ink modules have an orange "flex-connector", attatching the display to the PCB.
There is some amount of text printed on the connector.

![E-ink module, end-on, displaying flex connector label position](flex_connector.jpg)

The main label, printed on the connector, can be used to [identify the model](#supported-displays), 
or can be used, as a class name, to declare your panel in your sketch.

To use this label directly as a class name, you should remove all non-alphanumeric characters (no spaces, no dashes, no periods etc.) <br />
For the panel shown above, this class name would be `FPC7525`

```cpp
// Declaration by model
DEPG0154BNS800 display( PIN_DC, PIN_CS, PIN_BUSY );

// Is equivalent to

// Declaration by Flex Connector Label
FPC7525 display( PIN_DC, PIN_CS, PIN_BUSY );
```

## Wiring

**Do not connect display directly to Arduino UNO!**

*Arduino UNO* uses 5V signals, *Heltec E-ink displays* use 3.3V signals. Without a level shifting circuit, you will damage the display.

### Example of connection, using a voltage dividing level-shifter:

Schematic|Realization
---|---
![schematic of Arduino UNO, connected to display, through voltage dividing level shifter](/docs/Quickstart/schematic_voltage_divider.png) | ![graphic of Arduino UNO, connected to display, through voltage dividing level shifter](/docs/wiring_example.png)

### Display Pins:

* **VCC**: 3.3V Power In
* **GND**: Ground
* **D/C**: Data / Command
    * Tells display whether incoming serial data is a command, or is image data.
        * Can connect to any available digital pin on Arduino
        * 3.3V Logic Input, needs level shifter

* **SDI**: Serial Data Input
    * SPI *MOSI* pin
    * On Arduino UNO, *must connect to pin 11*
    * 3.3V Logic Input, needs level shifter
* **CS**: Chip Select
    * SPI *SS* Pin
    * Can connect to any available digital pin on Arduino, however *pin 10* is traditional. 
    * 3.3V Logic Input, needs level shifter
* **CLK**: Clock
    * SPI *SCK* pin
    * On Arduino UNO, *must connect to pin 13*
    * 3.3V Logic Input, needs level shifter
* **BUSY**
    * Pin is LOW when screen is ready to accept commands
    * Can connect to any available digital pin on Arduino
    * 3.3V Logic Output, **level shifter not required** as 3.3V is a valid level for a HIGH signal on Arduino UNO

Make sure to specify the location of your *D/C, CS* and *BUSY* pins in the constructor.

## Using the library

```c++
#include "heltec-eink-modules.h"

// Use the correct class for your display
DEPG0150BNS810 display(/* DC PIN */  8, /* CS PIN */  10, /* BUSY PIN */ 7);

void setup() {

    // All drawing commands go inside this WHILE
    while ( display.calculating() ) {
        display.fillCircle(50, 100, 20, display.colors.BLACK);
    }
    
    // Draw this new image to the display
    display.update();
}

void loop() {}
```

To summarize: 

* Set your hardware pins in the constructor
* All drawing commands go inside the ```while ( .calculating () )``` loop. <br />
    This loop repeats the commands for each little slice (page) of the screen, as many times as needed
* Call ```.update()``` to show this new image data on the screen

That's it! Everything else (should) be taken care of automatically.

### Drawing Commands

I'm going to direct you to [the official adafruit-gfx tutorial](https://learn.adafruit.com/adafruit-gfx-graphics-library/graphics-primitives) for information on the drawing commands. 

This implementation has a few small differences, including:

* The display will only support a limited set of colors:
    * ```.colors.BLACK```
    * ```.colors.WHITE```
    * ```.colors.RED```  (3-Color displays only*)

* The adafruit ```.fillScreen()``` method will work just fine, however it is more efficient to use ```.setDefaultColor()```, before the paging while loop.

* A few new handy methods have been added to help with layout:
    * `.bounds.full.`   ---   *Dimensions for the whole screen* <br />
                                         * `.left()` <br />
                            * `.right()` <br />
                            * `.top()` <br />
                            * `.bottom()` <br />
                            * `.centerX()` <br />
                            * `.centerY()` <br />
                            <br />
    * `bounds.window.`  ---   *Dimensions for the current window ([see here](#setting-a-window))* <br />
                                         * `.left()` <br />
                            * `.right()` <br />
                            * `.top()` <br />
                            * `.bottom()` <br />
                            * `.centerX()` <br />
                            * `.centerY()` <br />
                            <br />
    * `.setCursorTopLeft()`  --- *sets text-cursor position by upper-left corner value*
    * `.getTextWidth()` 
    * `.getTextHeight()` 

For a more complete description, **[read the API here.](/docs/API.md)**

As decided by the Adafruit library, the ancient *"XBitmap"* is the format of choice for pre-rendered graphics. Luckily, GIMP maintains good support for it. If you need a hint on how to use it, I have thrown together a [tutorial on preparing XBitmap images](XBitmapTutorial/README.md).


## Configuration Options

* [Pins](#pins)
* [Page Size](#page-size)
* [Power Management](#power-management)
* [Setting A Window](#setting-a-window)
* [Fast Mode (Partial Refresh)](#fast-mode-partial-refresh)
* [Code Readability](#code-readability)

### Pins

```c++
// Make sure to use the correct class for your display model
DEPG0150BNS810 display(dc, cs, busy);   
```

Pass the Arduino digital pin numbers where the *D/C*, *CS*, and *BUSY* pins from the display are connected.

### Page Size

It is possible to set the page size in the constructor. The optional ```page_height``` argument sets the number of rows. The default value is 20, meaning the display is calculated 20 rows at a time. Higher values use more RAM.

```c++
DEPG0150BNS810 display(dc, cs, busy, page_height); 
```


### Power Management ###

Many E-Ink displays are able to enter a "deep sleep" power-saving mode. In this state, the module is no longer listening to any commands we issue. Traditionally, a hardware reset pin is provided, however, with the *Heltec Modules*, this is not the case.

One work-around is to manually cycle power to the display ("turn it off and on again"). This does require additional circuitry.

A suitable method of controlling the power is shown here, using a PNP transistor and a FET based level-shifter:

![voltage-divider example](hardware_reset_example.png)


```c++
void reset() {
    digitalWrite(2, HIGH);      // PNP transistor, block current flow
    delay(500);                 // Wait for everything to power off
    digitalWrite(2, LOW);       // PNP transistor, allow current flow
    delay(500);                 // Wait for everything to power back up
}

void setup() {
    // Set the pins for the reset
    pinMode(2, OUTPUT);
    digitalWrite(2, LOW);   // PNP transistor, allow current flow

    while (display.calculating()) {
        display.print("hello world");
    }
    display.update();

    // Sleep, wait 8 seconds, and reset (wake up)
    display.deepSleep();    // Display ignores all commands now, until reset
    delay(8000);
    reset();

}
```



Note the use of ```.deepSleep()``` . This is the method which puts the display into power-saving. In this state, the current draw is very low, but the working memory is preserved. This is important, if you wish to wake the display and continue drawing from where you left off.

#### Do:

* **Feel free to avoid this whole section** <br />
If you don't ```deepSleep()```, you won't need a reset circuit.

* **Write your own ```reset()``` function** <br />
    It is not provided, as it must suit your individual circuit.

#### Don't:
* **Don't ```deepSleep()``` during fast-mode** <br />
    When you reset, your screen's contents won't be preserved

* **Don't use a [voltage dividing level shifter](#wiring)** <br />
    Enough current can pass through the signal lines to prevent the display from powering down. If this issue occurs, you can try ```digitalWrite()``` the pins in your ```reset()``` function.

* **Don't sleep by leaving display power disconnected** <br />
    *( Unless you really want to )* <br />
    The drawback is that the working memory will be lost. This is not an issue if, after power-on, you intend to ```clear()```, or redraw the entire screen.<br />
    If, instead, you go straight to *fastmode*, or ```setWindow()```, the display will show static. <br />
    If you do decide to sleep in this way, it is still a good idea to call ```.deepSleep()``` first. In some displays, it seems necessary to ensure that the image remains crisp.
    
<br />

### Setting a Window ###

By default, the entire screen contents will be erased and updated when ```while ( .calculating () ) ``` runs. Alternatively, it is possible to overwrite only a section of the screen, leaving the remainder unchanged. This section is referred to as a *window*.

```c++
display.setWindow( LEFT, TOP, WIDTH, HEIGHT );

while( display.calculating() )
{
    // Graphics commands go here e.g
    // display.drawRect(0, 0, 100, 100, display.colors.BLACK);
}
display.update();
```
If needed, make sure to call ```.setRotation()``` before ```.setWindow()```, as the library needs this information to properly place the window.

Be aware that, due to hardware limitations of the displays, windows may only be drawn with a width which is a multiple of 8. *(Note: this limitation does not rotate with the `.setRotation()` command)*.

It is possible to place a window so that it will have precisely the requested dimensions, however this is not strictly required as the library will automatically expand the window until it both satisfies the hardware limitations, and is large enough to encompass the requested region.

This automatic expansion can lead to a border around your graphics. One way to work around this issue is to use the dimensions calculated in `.bounds.window.*` (see [drawing](#drawing-commands)). 

```c++
display.drawRect( display.bounds.window.left(),
                  display.bounds.window.top(), 
                  display.bounds.window.width(),
                  display.bounds.window.height(), 
                  display.colors.BLACK );
```

*(That's a lot of typing for a value that you use so often..  - [yes, yes it is](#code-readability))*

These values are updated to reflect the true dimensions of the window, rather than the dimensions requested in `.calculating()`. This means that although your window won't be quite the size you asked for, you will at least be able to use its full width.

**Note that `.setRotation()` calls should go before `.setWindow()` calls.**

### Fast Mode (Partial Refresh)

E-Ink displays generally take several seconds to refresh. Some displays have a secondary mode, where the image updates much faster. This is known officially as a *"Partial Refresh"*. For the sake of user-friendliness, this library instead uses the descriptive term *"Fast Mode*".

The trade-off is that images drawn in fast mode are of a lower quality. The process may also be particularly difficult on the hardware. **You should make sure to exit fast mode when not in use.**

Not all displays support fast mode. Some displays may have a physical limitation, however with others, it seems that the technical settings have not yet been calculated and released.

To enter fast mode:
```c++
display.setFastmode( display.fastmode.ON );

while (display.calculating()) {
    // Graphics stuff here
}

// update() is called automatically in fast mode
```
Note that this state is somewhat temperamental. If you wish to call ```setWindow()```, or to return to ```fastmode.OFF```, you should use ```fastmode.FINALIZE```:
```c++
display.setFastmode( display.fastmode.FINALIZE );

while (display.calculating()) {
    // Graphics stuff here
}

// Also no need for update() here
```
Images drawn in ```fastmode.FINALIZE``` are preserved when moving window, or returning to ```fastmode.OFF```. <br />
Images drawn in ```fastmode.ON``` will not be preserved. 

When ```fastmode.FINALIZE``` has run, the display will automatically return to ```fastmode.OFF```.

### Code Readability ###

If you are like me, you might feel that all these long calls make your code "wordy":

```c++
display.drawRect( display.bounds.window.left(),
                  display.bounds.window.top(), 
                  display.bounds.window.width(),
                  display.bounds.window.height(), 
                  display.colors.BLACK );
```

As an alternative, declaring reusable "shortcuts" at the start of the code can really cut down on a lot of the bloat:

```c++
// Ugly mess here but
DEPG0150BNS810::Bounds::Window w = display.bounds.window;
DEPG0150BNS810::ColorList c = display.colors;
        
// Nice & clean here
display.drawRect( w.left(), w.top(), w.width(), w.height(), c.BLACK );
```



## Installation

**Arduino:** Library can be installed to Arduino IDE with *Sketch* -> *Include Library* -> *Add .Zip Library..*, or through the built-in Library Manager.

**Platform.io:** Available through the built-in library registry, or alternatively, can be installed by extracting the Zip file to the lib folder of your project.

## Acknowledgements

Display information from both [official Heltec sources](https://github.com/HelTecAutomation/e-ink), and  [GxEPD2](https://github.com/ZinggJM/GxEPD2)

Drawing functions provided by [GFX Root](https://github.com/ZinggJM/GFX_Root), which itself is a stripped down version of [Adafruit GFX](https://github.com/adafruit/Adafruit-GFX-Library).
